﻿namespace FogIndex

open System.Windows
open System.Windows.Controls
open System.Windows.Media

[<AutoOpen>]
module Collection =
    let toDoubleCollection xs =
        let collection = DoubleCollection()
        xs |> Seq.iter collection.Add
        collection

[<AutoOpen>]
module Resources =
    open System.Windows.Shapes

    let dashedBorder =
        Rectangle(
            Stretch=Stretch.Fill,
            RadiusX=6.0,
            RadiusY=6.0,
            Stroke=SolidColorBrush(Colors.Black),
            StrokeThickness=6.0,
            StrokeLineJoin=PenLineJoin.Round,
            StrokeDashCap=PenLineCap.Round,
            StrokeDashArray=([4.0;3.0] |> toDoubleCollection)
        )

    let solidBorder =
        Rectangle(
            Stretch=Stretch.Fill,
            RadiusX=6.0,
            RadiusY=6.0,
            Stroke=SolidColorBrush(Colors.LightGray),
            StrokeThickness=6.0,
            StrokeLineJoin=PenLineJoin.Round
        )

type Calculator () as this =
    inherit UserControl ()

    let mutable disposables = []
    let remember disposable = disposables <- disposable::disposables

    let rand = System.Random()
    let clouds = 
        [1..10]
        |> List.map (fun _ ->
            let x = rand.NextDouble() * 0.0
            let y = rand.NextDouble() * 300.0
            let e = System.Windows.Shapes.Ellipse()
            e.Effect <- Effects.BlurEffect()
            e.Fill <- RadialGradientBrush(Colors.LightGray, Colors.White)
            e.Width <- rand.NextDouble() ** 2.0 * 100.0 + 100.0
            e.Height <- rand.NextDouble() ** 2.0 * 50.0 + 50.0
            e.Opacity <- 0.25 + rand.NextDouble() * 0.50
            let canvas = Canvas()
            canvas.HorizontalAlignment <- HorizontalAlignment.Center
            canvas.Children.Add e
            canvas.RenderTransform <- TranslateTransform(X=x-e.Width/2.0,Y=y-e.Height/2.0)
            
            (rand.NextDouble()**2.0 - 0.5,rand.NextDouble()*0.1 - 0.05),canvas
        )

    let label = 
        TextBlock(
            Text="Fog Index",
            Foreground=SolidColorBrush(Colors.Gray),
            FontWeight=FontWeights.Bold,
            FontSize=18.0,
            Margin=Thickness(8.0,8.0,8.0,0.0)
        )
    let pasteText = 
        TextBox(Text="Paste Text Here",
                BorderThickness=Thickness(0.0),
                Margin=Thickness(8.0),
                AcceptsReturn=true,
                TextWrapping=TextWrapping.Wrap,
                SelectionBackground=SolidColorBrush(Colors.Gray),
                SelectionForeground=SolidColorBrush(Colors.White)
        )

    let makeBorder parent child =
        let border = Grid()
        do  border.Children.Add parent
        do  border.Children.Add child
        border

    let pasteBox = makeBorder dashedBorder pasteText
    do  pasteBox.Margin <- Thickness(8.0)

    let button = Button(Content="Paste Text From Clipboard")
    do  button.Margin <- Thickness(8.0,0.0,8.0,8.0)
    do  button.Click.Add(fun me -> pasteText.Text <- Clipboard.GetText())
    
    let removeSuffixes word =
        let suffixes = ["es";"ed";"ing"]
        suffixes
        |> Seq.fold (fun (word':string) suffix -> 
            if word'.EndsWith suffix then
                word'.Substring(0,word'.Length-suffix.Length)
            else
                word'
        ) word

    let toFogIndex text =
        let sentences = getSentences text
        let words = sentences |> Array.collect getWords
        let complexWords =
            words
            |> Array.filter (fun word -> word.Length>3)
            |> Array.map removeSuffixes            
            |> Array.filter (fun word -> countSyllables word >= 3)
        0.4 * ((float words.Length/float sentences.Length) + 
                (100.0 * float complexWords.Length/float words.Length))

    do  pasteText.TextChanged         
        |> Observable.map (fun _ -> pasteText.Text)
        |> Observable.onThreadPool
        |> Observable.map toFogIndex
        |> Observable.onDispatcher
        |> Observable.subscribe (fun index -> 
            label.Text <- sprintf "Fog Index %0.2f" index
        )
        |> remember

    let layout =
        TableLayout([],["32";"*";"32"]) 
            + label 
            + pasteBox
            + button
        |> makeBorder solidBorder
    
    do  clouds |> Seq.map snd |> Seq.iter layout.Children.Add

    do  this.Content <- layout

    let moveClouds () =
        clouds |> Seq.iter (fun ((dx,dy),cloud) ->
        let transform = cloud.RenderTransform :?> TranslateTransform
        transform.X <- transform.X + dx
        transform.Y <- transform.Y + dy
        cloud.RenderTransform <- transform
    )

    let invoke f = this.Dispatcher.BeginInvoke(fun _ -> f()) |> ignore

    do  new System.Threading.Timer((fun _ -> invoke moveClouds), null, 0, 50)
        |> remember

    interface System.IDisposable with
        member this.Dispose() = 
            disposables |> List.iter (fun x -> x.Dispose())
